import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Observable, mergeMap, map, catchError, of, tap } from 'rxjs';
import { User } from 'src/app/shared/models/user/user.model';
import { UsersService } from 'src/app/shared/services/users.service';
import { UserActionTypes } from './user.action.types';
import {
  UserActionUnion,
  GetAllUsersActionSuccess,
  GetAllUsersActionError,
  SearchUserActionSuccess,
  SearchUserActionError,
  GetUser_s_ProjectsSuccess,
  GetUser_s_ProjectsError,
  CreateFormUserAddActionError,
  CreateFormUserAddActionSuccess,
  SaveUserFormAddActionError,
  SaveUserFormAddActionSuccess,
  CreareFormUserEditActionActionError,
  CreareFormUserEditActionSuccess,
  DeleteUserActionError,
  DeleteUserActionSuccess,
  UpdateUserFormActionError,
  UpdateUserFormActionSuccess,
} from './users.actions';
@Injectable()
export class UsersEffects {
  effectActions: any;
  constructor(
    private userActions: Actions,
    private usersService: UsersService,
    private router: Router
  ) {}

  //create getAllUsers effect

  getAllUsersEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.GET_ALL_USERS),
      mergeMap(() => {
        return this.usersService.getAllUsers().pipe(
          map((users: User[]) => new GetAllUsersActionSuccess(users)),
          catchError((err) => of(new GetAllUsersActionError(err.message)))
        );
      })
    )
  );

  //create searchUser effect

  searchUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.SEARCH_A_USER),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.searchUser(userAction.payload).pipe(
          map((users: User[]) => new SearchUserActionSuccess(users)),
          catchError((err) => of(new SearchUserActionError(err.message)))
        );
      })
    )
  );

  // navigate to another component effect
  navigate: Observable<UserActionUnion> = createEffect(
    () =>
      this.userActions.pipe(
        ofType(UserActionTypes.NAVIGATE_TO_USER_S_PROJECTS_COMPONENT_SUCCESS),
        tap(() => {
          this.router.navigate(['/user-s-projects']);
        })
      ),
    { dispatch: false }
  );

  loadUserSProjectsEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.GET_USER_S_PROJECTS),
      mergeMap((action: UserActionUnion) => {
        return this.usersService.getUser_sProjects().pipe(
          map((users: any) => new GetUser_s_ProjectsSuccess(users)),
          catchError((err) => of(new GetUser_s_ProjectsError(err.message)))
        );
      })
    )
  );


   //create a user add form effect
   createAddUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
   this.userActions.pipe(
     ofType(UserActionTypes.CREATE_FORM_ADD_USER),
     map((userAction: UserActionUnion) => {
       return new CreateFormUserAddActionSuccess(userAction.payload);
     }),
     catchError((err) => of(new CreateFormUserAddActionError(err.message)))
   )
 );

 //saving user form data effect

 saveUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
   this.userActions.pipe(
     ofType(UserActionTypes.SAVE_USERFORM_ADD),
     mergeMap((userAction: UserActionUnion) => {
       return this.usersService.createUser(userAction.payload).pipe(
         map((user:User) => new SaveUserFormAddActionSuccess(user)),
         catchError((err) => of(new SaveUserFormAddActionError(err.message)))
       );
     })
   )
 );

 // create an update user form effect and fill it with data of a user to update

 createUpdateUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
 this.effectActions.pipe(
   ofType(UserActionTypes.CREATE_FORM_USER_EDIT),
   mergeMap((userAction: UserActionUnion) => {
     return this.usersService.getUserById(userAction.payload.id).pipe(
       map((user: User) => new CreareFormUserEditActionSuccess(user)),
       catchError((err) =>
         of(new CreareFormUserEditActionActionError(err.message))
       )
     );
   })
 )
);

// update a user with data of form effect

updateUserFormDataEffect: Observable<UserActionUnion> = createEffect(() =>
 this.effectActions.pipe(
   ofType(UserActionTypes.UPDATE_USER_FORM),
   mergeMap((userAction: UserActionUnion) => {
     return this.usersService.updateUser(userAction.payload).pipe(
       map((user: User) => new UpdateUserFormActionSuccess(user)),
       catchError((err) => of(new UpdateUserFormActionError(err.message)))
     );
   })
 )
);

//deleteUser effect

deleteUserEffect: Observable<UserActionUnion> = createEffect(() =>
 this.effectActions.pipe(
   ofType(UserActionTypes.DELETE_A_USER),
   mergeMap((userAction: UserActionUnion) => {
     return this.usersService.deleteUser(userAction.payload.id).pipe(
       map(() => new DeleteUserActionSuccess(userAction.payload)),
       catchError((err) => of(new DeleteUserActionError(err.message)))
     );
   })
 )
);

}
